@@ -1,7 +1,6 @@
 from agent import web_research_agent
 
 from inspect_ai import Task, task
-from inspect_ai.agent import bridge
 from inspect_ai.dataset import json_dataset
 from inspect_ai.scorer import model_graded_fact
 
@@ -10,6 +9,6 @@ from inspect_ai.scorer import model_graded_fact
 def research() -> Task:
     return Task(
         dataset=json_dataset("dataset.json"),
-        solver=bridge(web_research_agent()),
+        solver=web_research_agent(),
         scorer=model_graded_fact(),
     )
