@@ -43,6 +43,10 @@ export const ApplicationIcons = {
     remove: "bi bi-dash",
     replace: "bi bi-plus-slash-minus",
   },
+  checkbox: {
+    checked: "bi bi-check-circle",
+    unchecked: "bi bi-circle",
+  },
   chevron: {
     right: "bi bi-chevron-right",
     down: "bi bi-chevron-down",
@@ -56,6 +60,7 @@ export const ApplicationIcons = {
   config: "bi bi-gear",
   confirm: "bi bi-check",
   copy: "bi bi-copy",
+  display: "bi bi-card-text",
   epoch: (epoch: string) => {
     return `bi bi-${epoch}-circle`;
   },
@@ -100,6 +105,7 @@ export const ApplicationIcons = {
 
   next: "bi bi-chevron-right",
   noSamples: "bi bi-ban",
+  options: "bi bi-gear",
   play: "bi bi-play-fill",
   previous: "bi bi-chevron-left",
   refresh: "bi bi-arrow-clockwise",
