@@ -3,19 +3,19 @@ import JSON5 from "json5";
 import { FC, Fragment, isValidElement, JSX, ReactNode } from "react";
 import { ANSIDisplay } from "../../components/AnsiDisplay";
 import JSONPanel from "../../components/JsonPanel";
-import { MarkdownDiv } from "../../components/MarkdownDiv";
 import { formatNumber } from "../../utils/format";
 import { isJson } from "../../utils/json";
 import { ApplicationIcons } from "../appearance/icons";
 import { ChatMessageRenderer } from "../samples/chat/ChatMessageRenderer";
-import { MetaDataGrid } from "./MetaDataGrid";
-import styles from "./RenderedContent.module.css";
-import { Buckets, ContentRenderer, RenderOptions } from "./types";
 import {
   isMessageContent,
   MessageContent,
 } from "../samples/chat/MessageContent";
 import { defaultContext } from "../samples/chat/MessageContents";
+import { MetaDataGrid } from "./MetaDataGrid";
+import styles from "./RenderedContent.module.css";
+import { RenderedText } from "./RenderedText";
+import { Buckets, ContentRenderer, RenderOptions } from "./types";
 
 interface RenderedContentProps {
   id: string;
@@ -153,7 +153,7 @@ const contentRenderers: (
         const rendered = entry.value.trim();
         if (options.renderString === "markdown") {
           return {
-            rendered: <MarkdownDiv markdown={rendered} />,
+            rendered: <RenderedText markdown={rendered} />,
           };
         } else {
           return {
