@@ -15,15 +15,18 @@ import {
 } from "../../../@types/log";
 import { ContentTool } from "../../../app/types";
 import ExpandablePanel from "../../../components/ExpandablePanel";
-import { MarkdownDiv } from "../../../components/MarkdownDiv";
+import { isJson } from "../../../utils/json";
+
+import { RenderedText } from "../../content/RenderedText";
 import { ContentDataView } from "./content-data/ContentDataView";
+import { ContentDocumentView } from "./documents/ContentDocumentView";
+import { JsonMessageContent } from "./JsonMessageContent";
 import { MessageCitations } from "./MessageCitations";
 import styles from "./MessageContent.module.css";
 import { MessagesContext } from "./MessageContents";
+import { ServerToolCall } from "./server-tools/ServerToolCall";
 import { ToolOutput } from "./tools/ToolOutput";
 import { Citation } from "./types";
-import { ServerToolCall } from "./server-tools/ServerToolCall";
-import { ContentDocumentView } from "./documents/ContentDocumentView";
 
 type ContentObject =
   | ContentText
@@ -125,7 +128,7 @@ interface MessageRenderer {
 
 const messageRenderers: Record<string, MessageRenderer> = {
   text: {
-    render: (key, content, isLast) => {
+    render: (key, content, isLast, _context) => {
       // The context provides a way to share context between different
       // rendering. In this case, we'll use it to keep track of citations
       const c = content as ContentText;
@@ -135,26 +138,45 @@ const messageRenderers: Record<string, MessageRenderer> = {
         return undefined;
       }
 
-      return (
-        <>
-          <MarkdownDiv
-            key={key}
-            markdown={c.text || ""}
-            className={isLast ? "no-last-para-padding" : ""}
-          />
-          {c.citations ? (
-            <MessageCitations citations={c.citations as Citation[]} />
-          ) : undefined}
-        </>
-      );
+      const purgeInternalContainers = (text: string): string => {
+        // Remove any <internal>...</internal> tags and their contents
+        const internalTags = ["internal", "content-internal", "think"];
+        internalTags.forEach((tag) => {
+          const regex = new RegExp(`<${tag}[^>]*>[\\s\\S]*?<\\/${tag}>`, "gm");
+          text = text.replace(regex, "");
+        });
+
+        return text.trim();
+      };
+
+      if (isJson(c.text)) {
+        const obj = JSON.parse(c.text);
+        return <JsonMessageContent id={`${key}-json`} json={obj} />;
+      } else {
+        return (
+          <>
+            <RenderedText
+              key={key}
+              markdown={purgeInternalContainers(c.text) || ""}
+              className={isLast ? "no-last-para-padding" : ""}
+            />
+            {c.citations ? (
+              <MessageCitations citations={c.citations as Citation[]} />
+            ) : undefined}
+          </>
+        );
+      }
     },
   },
   reasoning: {
-    render: (key, content, isLast) => {
+    render: (key, content, isLast, _context) => {
       const r = content as ContentReasoning;
       if (!r.reasoning && !r.redacted) {
         return undefined;
       }
+      const text = r.redacted
+        ? "Reasoning encrypted by model provider."
+        : r.reasoning;
       return (
         <div key={key} className={clsx(styles.reasoning, "text-size-small")}>
           <div
@@ -167,20 +189,14 @@ const messageRenderers: Record<string, MessageRenderer> = {
             Reasoning
           </div>
           <ExpandablePanel id={`${key}-reasoning`} collapse={true}>
-            <MarkdownDiv
-              markdown={
-                r.redacted
-                  ? "Reasoning encrypted by model provider."
-                  : r.reasoning
-              }
-            />
+            <RenderedText markdown={text} />{" "}
           </ExpandablePanel>
         </div>
       );
     },
   },
   image: {
-    render: (key, content) => {
+    render: (key, content, _isLast, _context) => {
       const c = content as ContentImage;
       if (c.image.startsWith("data:")) {
         return <img src={c.image} className={styles.contentImage} key={key} />;
@@ -190,7 +206,7 @@ const messageRenderers: Record<string, MessageRenderer> = {
     },
   },
   audio: {
-    render: (key, content) => {
+    render: (key, content, _isLast, _context) => {
       const c = content as ContentAudio;
       return (
         <audio controls key={key}>
@@ -200,7 +216,7 @@ const messageRenderers: Record<string, MessageRenderer> = {
     },
   },
   video: {
-    render: (key, content) => {
+    render: (key, content, _isLast, _context) => {
       const c = content as ContentVideo;
       return (
         <video width="500" height="375" controls key={key}>
@@ -210,27 +226,27 @@ const messageRenderers: Record<string, MessageRenderer> = {
     },
   },
   tool: {
-    render: (key, content) => {
+    render: (key, content, _isLast, _context) => {
       const c = content as ContentTool;
       return <ToolOutput output={c.content} key={key} />;
     },
   },
   // server-side tool use
   tool_use: {
-    render: (key, content) => {
+    render: (key, content, _isLast, _context) => {
       const c = content as ContentToolUse;
       // If the tool use has a tool, render it
       return <ServerToolCall id={key} content={c} />;
     },
   },
   data: {
-    render: (key, content) => {
+    render: (key, content, _isLast, _context) => {
       const c = content as ContentData;
       return <ContentDataView id={key} contentData={c} />;
     },
   },
   document: {
-    render: (key, content) => {
+    render: (key, content, _isLast, _context) => {
       const c = content as ContentDocument;
       return <ContentDocumentView id={key} document={c} />;
     },
