@@ -1,6 +1,6 @@
 import { highlightElement } from "prismjs";
-import { useCallback, useEffect, useMemo, useRef } from "react";
-import { Events } from "../@types/log";
+import { RefObject, useCallback, useEffect, useMemo, useRef } from "react";
+import { EvalSample, EvalSpec, Events } from "../@types/log";
 import {
   createEvalDescriptor,
   createSamplesDescriptor,
@@ -14,6 +14,7 @@ import {
 import { LogFile, SampleSummary } from "../client/api/types";
 import { kEpochAscVal, kSampleAscVal, kScoreAscVal } from "../constants";
 import { createLogger } from "../utils/logger";
+import { prettyDirUri } from "../utils/uri";
 import { getAvailableScorers, getDefaultScorer } from "./scoring";
 import { useStore } from "./store";
 import { mergeSampleSummaries } from "./utils";
@@ -454,28 +455,27 @@ export const usePrevious = <T>(value: T) => {
 // Syntax highlighting strings larger than this is too slow
 const kPrismRenderMaxSize = 250000;
 
-export const usePrismHighlight = (toolCallContent?: string) => {
-  const toolViewRef = useRef<HTMLDivElement>(null);
-
+export const usePrismHighlight = (
+  containerRef: RefObject<HTMLDivElement | null>,
+  contentLength: number,
+) => {
   useEffect(() => {
-    if (
-      toolCallContent &&
-      toolViewRef.current &&
-      toolCallContent.length <= kPrismRenderMaxSize
-    ) {
-      requestAnimationFrame(() => {
-        const codeBlocks = toolViewRef.current?.querySelectorAll("pre code");
+    requestAnimationFrame(() => {
+      if (
+        contentLength > 0 &&
+        containerRef.current !== null &&
+        contentLength <= kPrismRenderMaxSize
+      ) {
+        const codeBlocks = containerRef.current?.querySelectorAll("pre code");
         codeBlocks?.forEach((block) => {
           if (block.className.includes("language-")) {
             block.classList.add("sourceCode");
             highlightElement(block as HTMLElement);
           }
         });
-      });
-    }
-  }, [toolCallContent]);
-
-  return toolViewRef;
+      }
+    });
+  }, [contentLength, containerRef]);
 };
 
 export const useSetSelectedLogIndex = () => {
@@ -553,6 +553,13 @@ export const useSamplePopover = (id: string) => {
   return {
     show,
     hide,
+    setShowing: (shouldShow: boolean) => {
+      if (shouldShow) {
+        show();
+      } else {
+        hide();
+      }
+    },
     isShowing,
   };
 };
@@ -675,3 +682,34 @@ export const useLogsListing = () => {
     setFilteredCount,
   };
 };
+
+export interface TitleContext {
+  logDir?: string;
+  evalSpec?: EvalSpec;
+  sample?: EvalSample;
+}
+
+export const useDocumentTitle = () => {
+  const setDocumentTitle = (context: TitleContext) => {
+    const title: string[] = [];
+
+    if (context.sample) {
+      title.push(`${context.sample.id}_${context.sample.epoch}`);
+    }
+
+    if (context.evalSpec) {
+      title.push(`${context.evalSpec.model} - ${context.evalSpec.task}`);
+    }
+
+    if (context.logDir) {
+      title.push(prettyDirUri(context.logDir));
+    }
+
+    if (title.length === 0) {
+      title.push("Inspect View");
+    }
+
+    document.title = title.join(" - ");
+  };
+  return { setDocumentTitle };
+};
