@@ -1,5 +1,7 @@
 from inspect_ai.hooks._hooks import (
     ApiKeyOverride,
+    EvalSetEnd,
+    EvalSetStart,
     Hooks,
     ModelUsageData,
     RunEnd,
@@ -15,6 +17,8 @@ __all__ = [
     "ApiKeyOverride",
     "Hooks",
     "ModelUsageData",
+    "EvalSetStart",
+    "EvalSetEnd",
     "RunEnd",
     "RunStart",
     "SampleEnd",
