@@ -1,5 +1,6 @@
 import json
 import logging
+import math
 import os
 import tempfile
 from logging import getLogger
@@ -8,13 +9,12 @@ from zipfile import ZIP_DEFLATED, ZipFile
 
 import anyio
 from pydantic import BaseModel, Field
-from pydantic_core import to_json
 from typing_extensions import override
 
 from inspect_ai._util.constants import DESERIALIZING_CONTEXT, LOG_SCHEMA_VERSION
 from inspect_ai._util.error import EvalError, WriteConflictError
 from inspect_ai._util.file import FileSystem, dirname, file, filesystem
-from inspect_ai._util.json import jsonable_python
+from inspect_ai._util.json import to_json_safe
 from inspect_ai._util.trace import trace_action
 
 from .._log import (
@@ -64,10 +64,12 @@ class EvalRecorder(FileRecorder):
         return location.endswith(".eval")
 
     @override
-    def default_log_buffer(self) -> int:
+    def default_log_buffer(self, sample_count: int) -> int:
         # .eval files are 5-8x smaller than .json files so we
         # are much less worried about flushing frequently
-        return 10
+        # scale flushes in alignment with sample_count so small runs
+        # flush more often (sample by sample) and large runs less often
+        return max(1, min(math.floor(sample_count / 3), 10))
 
     def __init__(self, log_dir: str, fs_options: dict[str, Any] = {}):
         super().__init__(log_dir, ".eval", fs_options)
@@ -185,19 +187,30 @@ class EvalRecorder(FileRecorder):
         # and then read it from a temp file (eliminates the possiblity of hundreds
         # of small fetches from the zip file streams)
         temp_log: str | None = None
+        etag: str | None = None
         fs = filesystem(location)
+
         if not fs.is_local() and header_only is False:
             with tempfile.NamedTemporaryFile(delete=False) as temp:
                 temp_log = temp.name
-                fs.get_file(location, temp_log)
+                if fs.is_s3():
+                    # download file and get ETag so it matches the content
+                    etag = await _s3_download_with_etag(location, temp_log, fs)
+                else:
+                    fs.get_file(location, temp_log)
 
         # read log (use temp_log if we have it)
         try:
             with file(temp_log or location, "rb") as z:
                 log = _read_log(z, location, header_only)
 
-                if fs.is_s3():
+                if etag is not None:
+                    log.etag = etag
+                elif fs.is_s3() and header_only:
                     file_info = fs.info(location)
+                    # if the file is modified in S3 at this point, the ETag will be incorrect
+                    # this is challenging to fix
+                    # but this should be ok because the ETag is for conditional writes, and only the entire log gets written back
                     log.etag = file_info.etag
 
                 return log
@@ -338,6 +351,37 @@ async def _s3_conditional_put_object(
         )
 
 
+async def _s3_download_with_etag(
+    location: str, local_path: str, fs: FileSystem
+) -> str | None:
+    """
+    Download S3 file and get its ETag in a single operation.
+
+    Returns:
+        ETag of the downloaded file (guaranteed to match the downloaded content)
+    """
+    import aioboto3
+
+    bucket, key = _s3_bucket_and_key(location)
+
+    session = aioboto3.Session()
+    async with session.client(
+        "s3",
+        endpoint_url=fs.fs.client_kwargs.get("endpoint_url"),
+        aws_access_key_id=fs.fs.key,
+        aws_secret_access_key=fs.fs.secret,
+        region_name=fs.fs.client_kwargs.get("region_name"),
+    ) as s3_client:
+        response = await s3_client.get_object(Bucket=bucket, Key=key)
+
+        content = await response["Body"].read()
+        with open(local_path, "wb") as f:
+            f.write(content)
+
+        etag: str = response["ETag"]
+        return etag.strip('"')  # S3 returns ETag with quotes
+
+
 async def _write_s3_conditional(
     fs: FileSystem,
     bucket: str,
@@ -482,12 +526,7 @@ class ZipLogFile:
         assert self._zip
         self._zip.writestr(
             filename,
-            to_json(
-                value=jsonable_python(data),
-                indent=2,
-                exclude_none=True,
-                fallback=lambda _x: None,
-            ),
+            to_json_safe(data),
         )
 
 
