@@ -20,7 +20,7 @@ class Recorder(abc.ABC):
     def handles_location(cls, location: str) -> bool: ...
 
     @abc.abstractmethod
-    def default_log_buffer(self) -> int: ...
+    def default_log_buffer(self, sample_count: int) -> int: ...
 
     @abc.abstractmethod
     def is_writeable(self) -> bool: ...
