@@ -45,6 +45,7 @@ from ._generate_config import (
     ResponseSchema,
 )
 from ._model import (
+    GenerateFilter,
     Model,
     ModelAPI,
     ModelName,
@@ -60,6 +61,8 @@ from ._model_output import (
     StopReason,
     TopLogprob,
 )
+from ._openai_convert import messages_from_openai, messages_to_openai
+from ._prompt import user_prompt
 from ._providers.providers import *
 from ._registry import modelapi
 from ._trim import trim_messages
@@ -68,6 +71,7 @@ __all__ = [
     "BatchConfig",
     "GenerateConfig",
     "GenerateConfigArgs",
+    "GenerateFilter",
     "ResponseSchema",
     "CachePolicy",
     "ContentAudio",
@@ -87,6 +91,8 @@ __all__ = [
     "ChatMessageAssistant",
     "ChatMessageTool",
     "ChatCompletionChoice",
+    "messages_from_openai",
+    "messages_to_openai",
     "ModelCall",
     "ModelOutput",
     "ModelConversation",
@@ -102,6 +108,7 @@ __all__ = [
     "execute_tools",
     "ExecuteToolsResult",
     "trim_messages",
+    "user_prompt",
     "cache_clear",
     "cache_list_expired",
     "cache_path",
