@@ -1,7 +1,7 @@
 from logging import getLogger
 from typing import Any, Literal, Type, Union
 
-from pydantic import BaseModel, Field, JsonValue, model_validator
+from pydantic import BaseModel, Field, model_validator
 from shortuuid import uuid
 
 from inspect_ai._util.constants import DESERIALIZING
@@ -44,9 +44,6 @@ class ChatMessageBase(BaseModel):
 
         return metadata_as(self.metadata, metadata_cls)
 
-    internal: JsonValue | None = Field(default=None)
-    """Model provider specific payload - typically used to aid transformation back to model types."""
-
     def model_post_init(self, __context: Any) -> None:
         # check if deserializing
         is_deserializing = isinstance(__context, dict) and __context.get(
