@@ -12,10 +12,12 @@ from types import TracebackType
 from typing import (
     Any,
     AsyncIterator,
+    Awaitable,
     Callable,
     Literal,
     Sequence,
     Type,
+    TypeAlias,
     cast,
 )
 
@@ -92,6 +94,16 @@ from ._model_output import ModelOutput, ModelUsage
 
 logger = logging.getLogger(__name__)
 
+GenerateFilter: TypeAlias = Callable[
+    [str, list[ChatMessage], list[ToolInfo], ToolChoice | None, GenerateConfig],
+    Awaitable[ModelOutput | None],
+]
+"""Filter a model generation.
+
+A filter may substitute for the default model generation by returning a
+`ModelOutput` or return `None` to allow default processing to continue.
+"""
+
 
 class ModelAPI(abc.ABC):
     """Model API provider.
@@ -1304,7 +1316,6 @@ def tool_result_images_reducer(
                     content=edited_tool_message_content,
                     tool_call_id=message.tool_call_id,
                     function=message.function,
-                    internal=message.internal,
                 )
             ],
             pending_content + new_user_message_content,
