@@ -1,6 +1,5 @@
-import base64
+import functools
 import json
-import re
 import socket
 from copy import copy
 from typing import Any, Literal
@@ -57,6 +56,11 @@ from inspect_ai._util.images import file_as_data_uri
 from inspect_ai._util.url import is_http_url
 from inspect_ai.model._call_tools import parse_tool_call
 from inspect_ai.model._generate_config import GenerateConfig
+from inspect_ai.model._internal import (
+    CONTENT_INTERNAL_TAG,
+    content_internal_tag,
+    parse_content_with_internal,
+)
 from inspect_ai.model._model_output import ChatCompletionChoice, Logprobs
 from inspect_ai.model._reasoning import parse_content_with_reasoning
 from inspect_ai.tool import ToolCall, ToolChoice, ToolFunction, ToolInfo
@@ -196,10 +200,16 @@ async def openai_chat_message(
         raise ValueError(f"Unexpected message role {message.role}")
 
 
-async def openai_chat_messages(
+async def messages_to_openai(
     messages: list[ChatMessage],
     system_role: Literal["user", "system", "developer"] = "system",
 ) -> list[ChatCompletionMessageParam]:
+    """Convert messages to OpenAI Completions API compatible messages.
+
+    Args:
+       messages: List of messages to convert
+       system_role: Role to use for system messages (newer OpenAI models use "developer" rather than "system").
+    """
     return [await openai_chat_message(message, system_role) for message in messages]
 
 
@@ -269,13 +279,9 @@ def openai_assistant_content(message: ChatMessageAssistant) -> str:
                 content = f"{content}\n<think{attribs}>\n{c.reasoning}\n</think>\n"
             elif c.type == "text":
                 content = f"{content}\n{c.text}"
+                if c.internal is not None:
+                    content = f"{content}\n<{content_internal_tag(c.internal)}>\n"
 
-    if message.internal:
-        content = f"""{content}\n<internal>{
-            base64.b64encode(json.dumps(message.internal).encode("utf-8")).decode(
-                "utf-8"
-            )
-        }</internal>\n"""
     return content
 
 
@@ -366,10 +372,19 @@ def chat_tool_calls_from_openai(
         return None
 
 
-def chat_messages_from_openai(
-    model: str,
+async def messages_from_openai(
     messages: list[ChatCompletionMessageParam],
+    model: str | None = None,
 ) -> list[ChatMessage]:
+    """Convert OpenAI Completions API messages into Inspect messages.
+
+    Args:
+        messages: OpenAI Completions API Messages
+        model: Optional model name to tag assistant messages with.
+    """
+    # some cleanup operations to compensate for various scaffolds
+    messages = functools.reduce(openai_assistant_message_reducer, messages, [])
+
     # track tool names by id
     tool_names: dict[str, str] = {}
 
@@ -398,17 +413,14 @@ def chat_messages_from_openai(
         elif message["role"] == "assistant":
             # resolve content
             refusal: Literal[True] | None = None
-            internal: JsonValue | None = None
             asst_content = message.get("content", None)
             if isinstance(asst_content, str):
-                # Even though the choices API doesn't take advantage of .internal,
-                # we could be transforming from OpenAI choices to Inspect for agent
-                # bridge scenarios where a different model (that does use .internal)
-                # is the actual model being used.
-                asst_content, internal = _parse_content_with_internal(asst_content)
                 asst_content, smuggled_reasoning = parse_content_with_reasoning(
                     asst_content
                 )
+                asst_content, content_internal = parse_content_with_internal(
+                    asst_content, CONTENT_INTERNAL_TAG
+                )
                 if smuggled_reasoning:
                     content = [
                         ContentReasoning(
@@ -416,7 +428,7 @@ def chat_messages_from_openai(
                             signature=smuggled_reasoning.signature,
                             redacted=smuggled_reasoning.redacted,
                         ),
-                        ContentText(text=asst_content),
+                        ContentText(text=asst_content, internal=content_internal),
                     ]
                 else:
                     content = asst_content
@@ -461,7 +473,6 @@ def chat_messages_from_openai(
                     tool_calls=tool_calls or None,
                     model=model,
                     source="generate",
-                    internal=internal,
                 )
             )
         elif message["role"] == "tool":
@@ -472,8 +483,7 @@ def chat_messages_from_openai(
                 # of it to support agent bridge scenarios. We have to strip that
                 # data. To be clear, if it's <think>, we'll strip the <think> tag,
                 # but the reasoning summary itself will remain in the content.
-                content, _ = _parse_content_with_internal(tool_content)
-                content, _ = parse_content_with_reasoning(content)
+                content, _ = parse_content_with_reasoning(tool_content)
             else:
                 content = []
                 for tc in tool_content:
@@ -491,6 +501,41 @@ def chat_messages_from_openai(
     return chat_messages
 
 
+def openai_assistant_message_reducer(
+    messages: list[ChatCompletionMessageParam], message: ChatCompletionMessageParam
+) -> list[ChatCompletionMessageParam]:
+    # some scaffolds (e.g. codex cli) split assistant messages with
+    # content and tool calls into:
+    #
+    #    assistant[tool_calls] -> tool -> assistant[content].
+    #
+    # unfortunately for the case of assistant messages that have
+    # reasoning embedded w/ <think> this breaks GPT-5 (which always
+    # wants the reasoning before tool calls). fix this up as required.
+    if (
+        # dealing with at least 2 existing messages
+        len(messages) >= 2
+        # new message is an assistant message w/ content and no tool calls
+        and message["role"] == "assistant"
+        and "tool_calls" not in message
+        and "content" in message
+        # last existing message is a tool result
+        and messages[-1]["role"] == "tool"
+        # second to last existing message is an assistant message w/ tool
+        # calls and no content
+        and messages[-2]["role"] == "assistant"
+        and "tool_calls" in messages[-2]
+        and messages[-2].get("content", None) is None
+    ):
+        # move content from new assistant message (which is not appended)
+        # to previous assistant message
+        messages[-2]["content"] = message["content"]
+    else:
+        messages.append(message)
+
+    return messages
+
+
 def tool_call_from_openai(tool_call: ChatCompletionMessageToolCallParam) -> ToolCall:
     assert tool_call["type"] == "function", '"custom" tool calls are not supported'
     return parse_tool_call(
@@ -509,6 +554,7 @@ def content_from_openai(
         content["type"] = list(content.keys())[0]  # type: ignore[arg-type]
     if content["type"] == "text":
         text = content["text"]
+        text, content_internal = parse_content_with_internal(text, CONTENT_INTERNAL_TAG)
         if parse_reasoning:
             content_text, reasoning = parse_content_with_reasoning(text)
             if reasoning:
@@ -518,12 +564,12 @@ def content_from_openai(
                         signature=reasoning.signature,
                         redacted=reasoning.redacted,
                     ),
-                    ContentText(text=content_text),
+                    ContentText(text=content_text, internal=content_internal),
                 ]
             else:
-                return [ContentText(text=text)]
+                return [ContentText(text=text, internal=content_internal)]
         else:
-            return [ContentText(text=text)]
+            return [ContentText(text=text, internal=content_internal)]
     elif content["type"] == "reasoning":  # type: ignore[comparison-overlap]
         return [ContentReasoning(reasoning=content["reasoning"])]
     elif content["type"] == "image_url":
@@ -714,40 +760,3 @@ class OpenAIAsyncHttpxClient(httpx.AsyncClient):
         )
 
         super().__init__(**kwargs)
-
-
-def _parse_content_with_internal(
-    content: str,
-) -> tuple[str, JsonValue | None]:
-    """
-    Extracts and removes a smuggled <internal>...</internal> tag from the content string, if present.
-
-    Note:
-        This OpenAI model does not natively use `.internal`. However, in bridge
-        scenarios—where output from a model that does use `.internal` is routed
-        through this code—such a tag may be present and should be handled.
-
-    Args:
-        content: The input string, possibly containing an <internal> tag with
-        base64-encoded JSON.
-
-    Returns:
-        tuple[str, JsonValue | None]:
-            - The content string with the <internal>...</internal> tag removed (if present), otherwise the original string.
-            - The decoded and parsed internal value (if present), otherwise None.
-
-    Raises:
-        json.JSONDecodeError: If the content of the <internal> tag is not valid JSON after decoding.
-        UnicodeDecodeError: If the content of the <internal> tag is not valid UTF-8 after base64 decoding.
-    """
-    internal_pattern = r"<internal>(.*?)</internal>"
-    internal_match = re.search(r"<internal>(.*?)</internal>", content, re.DOTALL)
-
-    return (
-        (
-            re.sub(internal_pattern, "", content, flags=re.DOTALL).strip(),
-            json.loads(base64.b64decode(internal_match.group(1)).decode("utf-8")),
-        )
-        if internal_match
-        else (content, None)
-    )
