@@ -192,10 +192,17 @@ class AzureAIAPI(ModelAPI):
         # create client (note the client needs to be created and closed
         # with each call so it can be cleaned up and not end up on another
         # event loop in a subsequent pass of eval)
-        assert self.api_key
+        if self.api_key is not None:
+            credential = AzureKeyCredential(self.api_key)
+        elif self.token_provider is not None:
+            credential = AzureKeyCredential(self.token_provider())
+        else:
+            raise PrerequisiteError(
+                "Azure AI must have either an API key or token provider."
+            )
         client = ChatCompletionsClient(
             endpoint=self.endpoint_url,
-            credential=AzureKeyCredential(self.api_key),
+            credential=credential,
             model=self.model_name,
             model_extras=self.model_args,
         )
