@@ -119,17 +119,18 @@ class OpenRouterAPI(OpenAICompatibleAPI):
 
         # provide openrouter standard reasoning options
         # https://openrouter.ai/docs/use-cases/reasoning-tokens
+        reasoning: dict[str, str | int] | None = None
         if config.reasoning_effort is not None or config.reasoning_tokens is not None:
-            reasoning: dict[str, str | int] = dict()
-            if config.reasoning_effort is not None:
-                reasoning["effort"] = config.reasoning_effort
+            reasoning = dict()
+            # openrouter supports one of max_tokens or effort, prefer max_tokens
             if config.reasoning_tokens is not None:
                 reasoning["max_tokens"] = config.reasoning_tokens
-            params["reasoning"] = reasoning
+            elif config.reasoning_effort is not None:
+                reasoning["effort"] = config.reasoning_effort
 
         # pass args if specifed
         EXTRA_BODY = "extra_body"
-        if self.models or self.provider or self.transforms:
+        if self.models or self.provider or self.transforms or reasoning:
             params[EXTRA_BODY] = params.get(EXTRA_BODY, {})
             if self.models:
                 params[EXTRA_BODY]["models"] = self.models
@@ -137,5 +138,7 @@ class OpenRouterAPI(OpenAICompatibleAPI):
                 params[EXTRA_BODY]["provider"] = self.provider
             if self.transforms:
                 params[EXTRA_BODY]["transforms"] = self.transforms
+            if reasoning:
+                params[EXTRA_BODY]["reasoning"] = reasoning
 
         return params
