@@ -1,5 +1,5 @@
 from contextlib import _AsyncGeneratorContextManager
-from typing import TypeAlias
+from typing import Callable, TypeAlias
 
 from anyio.streams.memory import MemoryObjectReceiveStream, MemoryObjectSendStream
 from mcp.shared.message import SessionMessage
@@ -8,5 +8,6 @@ MCPServerContext: TypeAlias = _AsyncGeneratorContextManager[
     tuple[
         MemoryObjectReceiveStream[SessionMessage | Exception],
         MemoryObjectSendStream[SessionMessage],
+        Callable[[], str | None],
     ],
 ]
