@@ -1,11 +1,10 @@
 import contextlib
-from types import TracebackType
 from typing import AsyncIterator, Sequence
 
 from .._tool import Tool, ToolSource
 from .._tool_def import ToolDef
 from ._types import MCPServer
-from .tools import MCPToolSource
+from .tools import MCPToolSourceLocal
 
 
 @contextlib.asynccontextmanager
@@ -29,32 +28,13 @@ async def mcp_connection(
     for tool_source in tool_sources:
         if isinstance(tool_source, MCPServer):
             mcp_servers.append(tool_source)
-        elif isinstance(tool_source, MCPToolSource):
+        elif isinstance(tool_source, MCPToolSourceLocal):
             mcp_servers.append(tool_source._server)
 
     # enter connection contexts
     async with contextlib.AsyncExitStack() as exit_stack:
-        for connection in [
-            MCPServerConnection(mcp_server) for mcp_server in mcp_servers
-        ]:
-            await exit_stack.enter_async_context(connection)
+        for mcp_server in mcp_servers:
+            await exit_stack.enter_async_context(mcp_server)
 
         # onward
         yield
-
-
-class MCPServerConnection:
-    def __init__(self, server: MCPServer) -> None:
-        self._server = server
-
-    async def __aenter__(self) -> "MCPServerConnection":
-        await self._server._connect()
-        return self
-
-    async def __aexit__(
-        self,
-        exc_type: type[BaseException] | None,
-        exc: BaseException | None,
-        exc_tb: TracebackType | None,
-    ) -> None:
-        await self._server._close()
