@@ -9,7 +9,7 @@ from types import TracebackType
 from typing import TYPE_CHECKING, Generic, Iterator, Literal, TypeVar
 
 import anyio
-from typing_extensions import Self
+from typing_extensions import Self, override
 
 from inspect_ai._util.logger import warn_once
 
@@ -194,6 +194,11 @@ class SampleLimits:
 
 def sample_limits() -> SampleLimits:
     """Get the top-level limits applied to the current `Sample`."""
+    # if there is _sample_limit_data recorded then the limit trees have
+    # gone out of scope for the sample so we just return that snapshot
+    limit_data = _sample_limit_data.get()
+    if limit_data is not None:
+        return limit_data
 
     def get_root_node(node: TNode | None, name: str) -> TNode:
         if node is None:
@@ -212,6 +217,23 @@ def sample_limits() -> SampleLimits:
     )
 
 
+def record_sample_limit_data(message_usage: float) -> None:
+    current_limits = sample_limits()
+    _sample_limit_data.set(
+        SampleLimits(
+            token=_LimitData(current_limits.token),
+            message=_LimitData(current_limits.message, usage=message_usage),
+            working=_LimitData(current_limits.working),
+            time=_LimitData(current_limits.time),
+        )
+    )
+
+
+_sample_limit_data: ContextVar[SampleLimits | None] = ContextVar(
+    "SampleLimitData", default=None
+)
+
+
 def token_limit(limit: int | None) -> _TokenLimit:
     """Limits the total number of tokens which can be used.
 
@@ -343,10 +365,54 @@ def record_waiting_time(waiting_time: float) -> None:
 
 
 def check_working_limit() -> None:
+    from inspect_ai.log._transcript import SampleLimitEvent, transcript
+
+    error = working_limit_exceeded()
+    if error is not None:
+        transcript()._event(
+            SampleLimitEvent(type="working", message=error.message, limit=error.limit)
+        )
+
+        raise error
+
+
+def monitor_working_limit(interval: float = 1) -> None:
+    from inspect_ai.log._samples import sample_active
+
+    # get the active sample
+    sample = sample_active()
+    if sample is None:
+        raise RuntimeError(
+            "monitor_working_limit() must be called from a running sample."
+        )
+    if sample.tg is None:
+        raise RuntimeError(
+            "monitor_working_limit() must be called after sample has been started."
+        )
+
+    # check every second
+    async def run() -> None:
+        while True:
+            await anyio.sleep(interval)
+
+            # don't continue after the sample is completed
+            if sample.completed:
+                return
+
+            error = working_limit_exceeded()
+            if error is not None:
+                sample.limit_exceeded(error)
+                return
+
+    # kick it off
+    sample.tg.start_soon(run)
+
+
+def working_limit_exceeded() -> LimitExceededError | None:
     node = working_limit_tree.get()
     if node is None:
-        return
-    node.check()
+        return None
+    return node.check()
 
 
 class _Tree(Generic[TNode]):
@@ -656,7 +722,7 @@ class _WorkingLimit(Limit, _Node):
             self.parent.record_waiting_time(waiting_time)
         self._waiting_time += waiting_time
 
-    def check(self) -> None:
+    def check(self) -> LimitExceededError | None:
         """Check if this working time limit or any ancestor limits have been exceeded.
 
         The checks occur from root to leaf. This is so that if multiple limits are
@@ -664,26 +730,25 @@ class _WorkingLimit(Limit, _Node):
         preventing certain sub-agent architectures from ending up in an infinite loop.
         """
         if self.parent is not None:
-            self.parent.check()
-        self._check_self()
-
-    def _check_self(self) -> None:
-        from inspect_ai.log._transcript import SampleLimitEvent, transcript
+            error = self.parent.check()
+            if error is not None:
+                return error
+        return self._check_self()
 
+    def _check_self(self) -> LimitExceededError | None:
         if self._limit is None:
-            return
+            return None
         if self.usage > self._limit:
             message = f"Working time limit exceeded. limit: {self._limit} seconds"
-            transcript()._event(
-                SampleLimitEvent(type="working", message=message, limit=self._limit)
-            )
-            raise LimitExceededError(
+            return LimitExceededError(
                 "working",
                 value=self.usage,
                 limit=self._limit,
                 message=message,
                 source=self,
             )
+        else:
+            return None
 
 
 def _validate_time_limit(name: str, value: float | None) -> None:
@@ -691,3 +756,34 @@ def _validate_time_limit(name: str, value: float | None) -> None:
         raise ValueError(
             f"{name} limit value must be a non-negative float or None: {value}"
         )
+
+
+class _LimitData(Limit):
+    """Limit which copies its values from another limit."""
+
+    def __init__(self, limit: Limit, *, usage: float | None = None) -> None:
+        self._limit = limit.limit
+        self._usage = usage if usage is not None else limit.usage
+
+    @override
+    def __enter__(self) -> Limit:
+        return self
+
+    @override
+    def __exit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> None:
+        pass
+
+    @property
+    @override
+    def limit(self) -> float | None:
+        return self._limit
+
+    @property
+    @override
+    def usage(self) -> float:
+        return self._usage
