@@ -1,6 +1,11 @@
 import pytest
-from test_helpers.utils import skip_if_no_together, skip_if_no_together_base_url
+from test_helpers.utils import (
+    skip_if_no_openai,
+    skip_if_no_together,
+    skip_if_no_together_base_url,
+)
 
+from inspect_ai._util.environ import environ_var
 from inspect_ai.model import (
     ChatMessageUser,
     GenerateConfig,
@@ -29,3 +34,13 @@ async def test_openai_compatible() -> None:
     message = ChatMessageUser(content="This is a test string. What are you?")
     response = await model.generate(input=[message])
     assert len(response.completion) >= 1
+
+
+@pytest.mark.asyncio
+@skip_if_no_openai
+async def test_openai_responses_compatible() -> None:
+    with environ_var("OPENAI_BASE_URL", "https://api.openai.com/v1"):
+        model = get_model("openai-api/openai/gpt-5", responses_api=True)
+        message = ChatMessageUser(content="This is a test string. What are you?")
+        response = await model.generate(input=[message])
+        assert len(response.completion) >= 1
