@@ -37,7 +37,7 @@ async def test_grok_num_choices() -> None:
 @pytest.mark.anyio
 @skip_if_no_together
 async def test_together_num_choices() -> None:
-    await check_num_choices("together/google/gemma-2b-it")
+    await check_num_choices("together/meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo")
 
 
 @pytest.mark.anyio
