@@ -13,6 +13,17 @@ async def test_number_eol():
     assert result.text == CORRECT
 
 
+@pytest.mark.anyio
+async def test_non_number_eol_markdown():
+    scorer = match(numeric=True)
+    state = simple_task_state(
+        model_output="**ANSWER: 28 + 32 = 60**\nThis solves the problem."
+    )
+    result = await scorer(state, Target(["60"]))
+
+    assert result.text == CORRECT
+
+
 @pytest.mark.anyio
 async def test_non_numeric_match():
     scorer = match(numeric=True, location="any")
