@@ -1,4 +1,4 @@
-from inspect_ai._util.file import basename
+from inspect_ai._util.file import basename, filesystem
 
 
 def test_basename():
@@ -14,3 +14,11 @@ def test_basename():
     assert basename(f"/opt/files/{MYDIR}/") == MYDIR
     assert basename(f"C:\\Documents\\{MYDIR}") == MYDIR
     assert basename(f"C:\\Documents\\{MYDIR}\\") == MYDIR
+
+
+def test_filesystem_file_info():
+    memory_filesystem = filesystem("memory://")
+    memory_filesystem.touch("test_file")
+    info = memory_filesystem.info("test_file")
+    assert info.name == "memory:///test_file"
+    assert info.size == 0
